/*
 * Copyright 2004 Berend "Kirk" Wouda
 * 
 * This file is part of WhatDoYouSee'd.
 * 
 * WhatDoYouSee'd is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * WhatDoYouSee'd is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with WhatDoYouSee'd; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


// The package, d'oh.
package WDYSEncrypt;

// Import the IO classes.
import java.io.*;

// Import the help class.
import system.HelperMethods;

/**
 * This class writes encrypted WDYS files, including a correct header and data
 * padding.
 * 
 * @author Berend "Kirk" Wouda
 * @version 2.10
 * @since 1.00
 */
public class WDYSEncryptedFileWriter {
	/**
	 * Constructs a new file writer around the given file.
	 * 
	 * @param filename The name of the encrypted file.
	 * @param length The length of the decypted data file, which we need for storing.
	 */
	public WDYSEncryptedFileWriter(String filename, long length) throws IOException {
		// Construct a new BufferedOutputStream around this baby.
		// Well... Put in a FileInputStream just to be certain ;^)
		writer = new BufferedOutputStream(new FileOutputStream(filename));
		
		// Construct an OutputStreamWriter over the stream. This will enable us to
		// write whatdoyousee in ASCII to the file :P
		OutputStreamWriter osr = new OutputStreamWriter(writer, "US-ASCII");
		
		// Write the header to the file. It consists of two parts, "whatdoyousee" and
		// an unsigned int indicating the length of the decrypted data.
		// First, whatdoyousee.
		osr.write("whatdoyousee");
		
		// Flush it to the file.
		osr.flush();
		
		// Next, the length attribute. This is a bit harder, but the helper class
		// provides... well, help. It converts the long to the correct format for us.
		writer.write(HelperMethods.convertLongToByteArray(length));
	}
	
	
	/**
	 * Closes the file.
	 * 
	 * @throws IOException When IO stuff goes wrong.
	 */
	public void close() throws IOException {
		// Yeah baby!
		writer.close();
	}
	
	/**
	 * Writes the next 8 bytes to the file.
	 * 
	 * @param octet The next 8 bytes to be written.
	 * @throws IOException When IO stuff goes wrong.
	 */
	public void write8EncryptedDataBytes(byte[] octet) throws IOException {
			// Read. I mean write.
			writer.write(octet, 0, 8);
	}
	
	
	/**
	 * The writer. Well it's a stream actually.
	 */
	protected BufferedOutputStream writer;
}