/*
 * Copyright 2004 Berend "Kirk" Wouda
 * 
 * This file is part of WhatDoYouSee'd.
 * 
 * WhatDoYouSee'd is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * WhatDoYouSee'd is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with WhatDoYouSee'd; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


// The package, I presume?
package WDYSEncrypt;

// Import the IO classes.
import java.io.*;

// Import the helper class.
import system.HelperMethods;

/**
 * This class encrypts the decrypted WDYS data, 2 times 4 bytes at a time.
 * 
 * @author Berend "Kirk" Wouda
 * @version 2.10
 * @since 1.00
 */
public class WDYSEncrypter {
	/**
	 * Constructs a new decrypter.
	 */
	public WDYSEncrypter() {
		// Uhm...
	}
	
	
	/**
	 * Encrypts the given 2x4 bytes into 8 bytes of encrypted data.
	 * 
	 * @param first The first 4 bytes to encrypt.
	 * @param second The second 4 bytes to encrypt.
	 * @return The decrypted 8 bytes.
	 */
	public byte[] encryptQuad(byte[] first, byte[] second) throws IOException {
		// What follows now is taken directly from a C example publicly posted by
		// someone anonymous.
		// I have no idea what it does exactly really, so I can't provide any useful
		// comments on it.
		// Oh dear lord this gets ugly. Java, you're so ugly when it comes to this.
		
		// Initialisation. I have no idea what the values stand for... most likely
		// some randomly chosen encryption stuff.
		// All values are longs. In the C code they were unsigned ints, but Java
		// doesn't have support for unsigned ints, so I used longs instead since they
		// have the same bit structure as unsigned ints in the unsigned int range.
		// What appears to be some base value. Don't forget the l suffix, d'oh!
		long base = 0x0l;
		
		// Two temporary values.
		long temp1;
		long temp2;
		
		// A crossreference table...
		long[] crossreference = new long[] {
			0x6c0a5452l,
			0x03827d0fl,
			0x3a170b92l,
			0x16db7fc2l
		};
		
		// Convert the passed unsigned int byte arrays into java longs.
		long out1 = HelperMethods.convertByteArrayToLong(first);
		long out2 = HelperMethods.convertByteArrayToLong(second);
		
		// Now the real decryption work starts. I have no idea what exactly goes on
		// in terms of algorithm, it's all just binary manipulation.
		// Note that there a number of AND's that cut away all the upper half data of
		// the long after an addition/subtraction. This is to mimic unsigned ints.
		// I added these myself, the rest is taken directly from the C code. I marked
		// them.
		for(int index = 0; index < 32; index++) {
			// Second word.
			temp1 = out2;
			temp2 = out2;
			
			temp1 >>= 5;
			temp2 <<= 4;
			temp1 ^= temp2;
			temp1 += out2;
			temp1 &= 0xFFFFFFFFl;	// Added to "cast" to an unsigned int.

			temp2 = base;
			temp2 &= 0x03l;

			temp2 = crossreference[(int) temp2];
			temp2 += base;
			base -= 0x61c88647l;

			temp1 ^= temp2;
			out1 += temp1;
			out1 &= 0xFFFFFFFFl;	// Added to "cast" to an unsigned int.

			// First word.
			temp1 = out1;
			temp2 = out1;
			
			temp1 >>= 5;
			temp2 <<= 4;
			temp1 ^= temp2;
			temp1 += out1;
			temp1 &= 0xFFFFFFFFl;	// Added to "cast" to an unsigned int.

			temp2 = base;
			temp2 >>= 0x0bl;
			temp2 &= 0x03l;

			temp2 = crossreference[(int) temp2];
			temp2 += base;

			temp1 ^= temp2;
			out2 += temp1;
			out2 &= 0xFFFFFFFFl;	// Added to "cast" to an unsigned int.
		}
		
		// Now we need to convert it all back in order to be able to write it
		// correctly to the output file.
		byte[] first4bytes = HelperMethods.convertLongToByteArray(out1);
		byte[] second4bytes = HelperMethods.convertLongToByteArray(out2);
		
		// Now we only need to copy both 4 bytes into a single 8 byte array...
		byte[] encrypted = new byte[8];
		System.arraycopy(first4bytes, 0, encrypted, 0, 4);
		System.arraycopy(second4bytes, 0, encrypted, 4, 4);
		
		// ...and return that.
		return encrypted;
	}
}